@@ -22,18 +22,30 @@
 #include "task.h"
 #include "message.h"
 
-int rspamd_smtp_received_parse (struct rspamd_task *task,
-		const char *data, size_t len, struct received_header *rh);
-int rspamd_smtp_addr_parse (const char *data, size_t len,
-		struct rspamd_email_address *addr);
-gboolean rspamd_content_disposition_parser (const char *data, size_t len,
-		struct rspamd_content_disposition *cd, rspamd_mempool_t *pool);
+
+#ifdef __cplusplus
+extern "C" {
+#endif
+
+int rspamd_smtp_addr_parse(const char *data, size_t len,
+						   struct rspamd_email_address *addr);
+
+gboolean rspamd_content_disposition_parser(const char *data, size_t len,
+										   struct rspamd_content_disposition *cd,
+										   rspamd_mempool_t *pool);
 
 gboolean
-rspamd_rfc2047_parser (const gchar *in, gsize len, gint *pencoding,
-		const gchar **charset, gsize *charset_len,
-		const gchar **encoded, gsize *encoded_len);
+rspamd_rfc2047_parser(const char *in, gsize len, int *pencoding,
+					  const char **charset, gsize *charset_len,
+					  const char **encoded, gsize *encoded_len);
 
-guint64 rspamd_parse_smtp_date (const char *data, size_t len);
+rspamd_inet_addr_t *rspamd_parse_smtp_ip(const char *data, size_t len,
+										 rspamd_mempool_t *pool);
+
+uint64_t rspamd_parse_smtp_date(const unsigned char *data, size_t len, GError **err);
+
+#ifdef __cplusplus
+}
+#endif
 
 #endif /* SRC_LIBMIME_SMTP_PARSERS_H_ */
